/**
 * 
 */
package clime.messadmin.providers.userdata;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import clime.messadmin.model.Application;
import clime.messadmin.model.RequestInfo;
import clime.messadmin.model.Server;
import clime.messadmin.model.Session;
import clime.messadmin.providers.ProviderUtils;
import clime.messadmin.providers.spi.ApplicationDataProvider;
import clime.messadmin.providers.spi.BaseTabularDataProvider;
import clime.messadmin.providers.spi.SessionDataProvider;
import clime.messadmin.providers.spi.SizeOfProvider;
import clime.messadmin.servletstats.RequestInfoHitsComparator;
import clime.messadmin.servletstats.Utils;
import clime.messadmin.utils.BytesFormat;

/**
 * @author C&eacute;drik LIME
 */
public class ServletStatsDisplayer extends BaseTabularDataProvider
		implements ApplicationDataProvider, SessionDataProvider {

	/**
	 * 
	 */
	public ServletStatsDisplayer() {
		super();
	}

	/****************************************************************/
	/*							Common methods						*/
	/****************************************************************/

	public String[] getTabularDataLabels() {
		return new String[] {"URL", "# errors", "# hits", "Mean Used Time", "Std Dev. Used Time", "Total Used Time"};
	}

	protected String getTableCaption(String[] labels, Object[][] values) {
		return ""+values.length+" servlet requests";
	}

	protected String getDataTitle(Map data) {
		String result = "<span title=\"";
        long currentItemSize = -1;
        Iterator iterProv = ProviderUtils.getProviders(SizeOfProvider.class).iterator();
        while (currentItemSize < 0 && iterProv.hasNext()) {
			SizeOfProvider provider = (SizeOfProvider) iterProv.next();
			currentItemSize = provider.sizeof(data);
		}
		result += "data size: " + BytesFormat.getBytesInstance().format(currentItemSize);
		result += "\">Servlets Statistics</span>";
		return result;

	}

	protected Object[][] getTabularData(Map data) {
		NumberFormat numberFormatter = NumberFormat.getNumberInstance();
		List inputList = new ArrayList(data.values());
		List resultList = new ArrayList(data.size());
		// sort data by #hits
		Collections.sort(inputList, new RequestInfoHitsComparator());
		Iterator iter = inputList.iterator();
		while (iter.hasNext()) {
			RequestInfo request = (RequestInfo) iter.next();
			resultList.add(new Object[] {
				request.getURL(),
				request.getNErrors()>0 ? "<span style=\"color: red; font-weight: bolder;\">" + numberFormatter.format(request.getNErrors()) +  "</span>" : numberFormatter.format(request.getNErrors()),
				numberFormatter.format(request.getHits()),
				numberFormatter.format(request.getMeanUsedTime()) + " ms",
				numberFormatter.format(request.getStdDevUsedTime()) + " ms",
				numberFormatter.format(request.getTotalUsedTime()) + " ms"
			});
		}
		Object[][] result = new Object[resultList.size()][];
		return (Object[][]) resultList.toArray(result);
	}

	/**
	 * {@inheritDoc}
	 */
	public int getPriority() {
		return 0;
	}

	/****************************************************************/
	/*							Session methods						*/
	/****************************************************************/

	public Object[][] getSessionTabularData(HttpSession httpSession) {
		Session session = Server.getInstance().getApplication(httpSession.getServletContext()).getSession(httpSession.getId());
		Map data = new HashMap(Utils.getPluginData(session));
		Object[][] result = getTabularData(data);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getSessionDataTitle(HttpSession httpSession) {
		Session session = Server.getInstance().getApplication(httpSession.getServletContext()).getSession(httpSession.getId());
		Map data = new HashMap(Utils.getPluginData(session));
		String result = getDataTitle(data);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getXHTMLSessionData(HttpSession session) {
		try {
			String[] labels = getTabularDataLabels();
			Object[][] values = getSessionTabularData(session);
			return buildXHTML(labels, values, "extraSessionAttributesTable-"+getClass().getName(), getTableCaption(labels, values));
		} catch (RuntimeException rte) {
			return "Error in " + this.getClass().getName() + ": " + rte;
		}
	}

	/****************************************************************/
	/*						Application methods						*/
	/****************************************************************/

	public Object[][] getApplicationTabularData(ServletContext context) {
		Application application = Server.getInstance().getApplication(context);
		Map data = new HashMap(Utils.getPluginData(application));
		Object[][] result = getTabularData(data);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getApplicationDataTitle(ServletContext context) {
		Application application = Server.getInstance().getApplication(context);
		Map data = new HashMap(Utils.getPluginData(application));
		String result = getDataTitle(data);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getXHTMLApplicationData(ServletContext context) {
		try {
			String[] labels = getTabularDataLabels();
			Object[][] values = getApplicationTabularData(context);
			return buildXHTML(labels, values, "extraApplicationAttributesTable-"+getClass().getName(), getTableCaption(labels, values));
		} catch (RuntimeException rte) {
			return "Error in " + this.getClass().getName() + ": " + rte;
		}
	}

}
